SystemOrganization addCategory: #'RPackage-SystemIntegration'!

!RPackage methodsFor: '*RPackage-SystemIntegration' stamp: 'StephaneDucasse 7/30/2010 22:45'!
importCategory
	"import all the classes of the class categories matching the receiver name (following package info semantics)." 
	
	| cat classesInCat |
	cat := Smalltalk organization categories select: [:each | each beginsWith:  self name ]. 
	classesInCat := cat gather: [:c | Smalltalk organization listAtCategoryNamed: c].
	classesInCat do: [:each | 
						self importClass: (Smalltalk globals at: each).
						self importClass: (Smalltalk globals at: each) class]! !

!RPackage methodsFor: '*RPackage-SystemIntegration' stamp: 'StephaneDucasse 7/29/2010 21:14'!
importCategory: aCat forClass: aClass 
	"import all the methods of a category as defined in the receiver." 
	
		(aClass organization listAtCategoryNamed: aCat)
			do: [:each | self addMethod: (aClass >> each)]
	
	! !

!RPackage methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrilledelaunay 1/24/2011 10:55'!
importClass: aClass 
	"import a class already created but not attached to a package to the receiver. 
	Handle also *- convention. Methods defined in *category are not added to the package."
	
	| protocols |
	self addClassDefinition: aClass.
	protocols := aClass organization categories 
		reject: [:each | (each first = $*) and: [ ( '*', self name asLowercase match: each asLowercase) not] ].
		"we reject all extensions except the ones that have the same name as the package - highly improbable but who knows"
	protocols do: [:each | self importProtocol: each forClass: aClass]
	
	! !

!RPackage methodsFor: '*RPackage-SystemIntegration' stamp: 'StephaneDucasse 7/29/2010 21:16'!
importProtocol: aProtocol forClass: aClass 
	"import all the methods of a protocol as defined in the receiver." 
	
		(aClass organization listAtCategoryNamed: aProtocol)
			do: [:each | self addMethod: (aClass >> each)]
	
	! !

!RPackageOrganizer class methodsFor: '*RPackage-SystemIntegration' stamp: 'StephaneDucasse 7/6/2010 13:39'!
isPackageDefinedForClass: aClass

	^ (RPackage organizer packageOf: aClass) notNil! !

!RPackageOrganizer class methodsFor: '*RPackage-SystemIntegration' stamp: 'StephaneDucasse 7/6/2010 13:41'!
methodAdded: anEvent
	"precondition: package exist, class exist"
	
	"| cm |
	cm := anEvent item.
	cm isValid ifFalse: [^self]. " "to eliminate comment and doit crap"
	| methodCategory |
	methodCategory := anEvent itemProtocol.
	(methodCategory isEmptyOrNil or:[methodCategory first ~= $*]) 
		ifTrue: [ ]
		ifFalse: [
					(self isPackageDefinedForClass: anEvent itemClass)
						ifTrue: []
						ifFalse: [RPackage new named: (Smalltalk organization categoryOfElement: anEvent itemClass asSymbol) ]
					.
	].
	! !

!RPackageOrganizer class methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrille.delaunay 11/26/2010 10:23'!
registerInterestToSystemAnnouncement

	self default registerInterestToSystemAnnouncement! !

!RPackageOrganizer class methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrille.delaunay 11/26/2010 10:23'!
unregisterInterestToSystemAnnouncement

	self default unregisterInterestToSystemAnnouncement! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrille.delaunay 11/25/2010 13:39'!
packageExactlyMatchingExtensionName: anExtensionName

	"only look for a package for which the name match 'anExtebsionName', making no difference about case. Retunr nil if no package is found"
	|tmpPackage tmpPackageName|
	packages keys do: [:aSymbol |
		(aSymbol sameAs: anExtensionName asSymbol)
			ifTrue: [
				tmpPackage := self packageNamed: aSymbol.
				]
		].

	^ tmpPackage! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrille.delaunay 11/25/2010 13:40'!
packageMatchingExtensionName: anExtensionName

	"return nil if no package i found"
	|tmpPackage tmpPackageName|
	"we first look if their is a package matching exactly the name specified"
	packages keys do: [:aSymbol |
		(aSymbol sameAs: anExtensionName asSymbol)
			ifTrue: [^ self packageNamed: aSymbol]
		].
	
	"if no package was found, we try to find one matching the begining of the name specified"
	tmpPackageName := ''.
	packages keys do: [:aSymbol |
		(aSymbol asString,'*'  match: anExtensionName)
			ifTrue: [
				"we keep the longest package name found"
				(aSymbol size > tmpPackageName size) ifTrue: [
					tmpPackageName := aSymbol.
					tmpPackage := self packageNamed: aSymbol
					].
				]
		].
	^ tmpPackage! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrille.delaunay 11/18/2010 12:50'!
registerInterestToAnnouncer: anAnnouncer
	"self registerInterestToSystemAnnouncement"

	"RPackageOrganizer registerInterestToSystemAnnouncement"

	"SystemAnnouncer initialize"

	anAnnouncer
		when: SystemCategoryAddedAnnouncement do: [ :ann | self systemCategoryAddedActionFrom: ann ];
		when: SystemCategoryRemovedAnnouncement do: [ :ann | self systemCategoryRemovedActionFrom: ann ];
		when: SystemCategoryRenamedAnnouncement do: [ :ann | self systemCategoryRenamedActionFrom: ann ];
		when: SystemClassAddedAnnouncement do: [ :ann | self systemClassAddedActionFrom: ann ];
		when: SystemClassCommentedAnnouncement
			do: [ "I don't see what should be done here'"];
		when: SystemClassRecategorizedAnnouncement do: [ :ann | self systemClassRecategorizedACtionFrom: ann ];
		when: SystemClassRemovedAnnouncement do: [ :ann | self systemClassRemovedActionFrom: ann ];
		when: SystemClassRenamedAnnouncement do: [ :ann | self systemClassRenamedActionFrom: ann ];
		when: SystemClassReorganizedAnnouncement do: [:ann | self systemClassReorganizedActionFrom: ann ];
		when: SystemMethodAddedAnnouncement do: [:ann | self systemMethodAddedActionFrom: ann  ];
		when: SystemMethodModifiedAnnouncement do: [:ann | self systemMethodModifiedActionFrom: ann  ];
		when: SystemMethodRecategorizedAnnouncement do: [:ann | self systemMethodRecategorizedActionFrom: ann  ];
		when: SystemMethodRemovedAnnouncement do: [:ann | self systemMethodRemovedActionFrom: ann  ];
		when: SystemProtocolAddedAnnouncement do: [:ann | self systemProtocolAddedActionFrom: ann  ];
		when: SystemProtocolRemovedAnnouncement do: [:ann | self systemProtocolRemovedActionFrom: ann  ]! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrilledelaunay 1/20/2011 17:23'!
registerInterestToSystemAnnouncement
	"self registerInterestToSystemAnnouncement"

	"RPackageOrganizer registerInterestToSystemAnnouncement"

	"SystemAnnouncer initialize"

	(SystemAnnouncer announcer)
		when: SystemCategoryAddedAnnouncement do: [ :ann | self systemCategoryAddedActionFrom: ann ];
		when: SystemCategoryRemovedAnnouncement do: [ :ann | self systemCategoryRemovedActionFrom: ann ];
		when: SystemCategoryRenamedAnnouncement do: [ :ann | self systemCategoryRenamedActionFrom: ann ];
		when: SystemClassAddedAnnouncement do: [ :ann | self systemClassAddedActionFrom: ann ];
		when: SystemClassCommentedAnnouncement
			do: [:ann | "I don't see what should be done here'"];
		when: SystemClassRecategorizedAnnouncement do: [ :ann | self systemClassRecategorizedACtionFrom: ann ];
		when: SystemClassRemovedAnnouncement do: [ :ann | self systemClassRemovedActionFrom: ann ];
		when: SystemClassRenamedAnnouncement do: [ :ann | self systemClassRenamedActionFrom: ann ];
		when: SystemClassReorganizedAnnouncement do: [:ann | self systemClassReorganizedActionFrom: ann ];
		when: SystemMethodAddedAnnouncement do: [:ann | self systemMethodAddedActionFrom: ann  ];
		when: SystemMethodModifiedAnnouncement do: [:ann | self systemMethodModifiedActionFrom: ann  ];
		when: SystemMethodRecategorizedAnnouncement do: [:ann | self systemMethodRecategorizedActionFrom: ann  ];
		when: SystemMethodRemovedAnnouncement do: [:ann | self systemMethodRemovedActionFrom: ann  ];
		when: SystemProtocolAddedAnnouncement do: [:ann | self systemProtocolAddedActionFrom: ann  ];
		when: SystemProtocolRemovedAnnouncement do: [:ann | self systemProtocolRemovedActionFrom: ann  ]! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrille.delaunay 11/25/2010 14:48'!
systemCategoryAddedActionFrom: ann

	| rPackage packageSymbol |
	"self flag: 'Seems to be ok'."
	packageSymbol := ann item asSymbol.
	(self includesPackageNamed: packageSymbol)
		ifFalse: [ 
			rPackage := RPackage named: packageSymbol.
			self registerPackage: rPackage ]! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'RPackage 1/20/2011 13:19'!
systemCategoryRemovedActionFrom: ann

	| rPackage categoryNameSymbol |
	self flag: 'should test if a MCPackage still exist. In this case, it should not remove the RPackage'.
	categoryNameSymbol := ann event item asSymbol.
	"here we check that a corresponding MCPackage does not still exist. In this case we should keep the RPackage"
	MCWorkingCopy allManagers detect: [:each | each packageName = categoryNameSymbol asString] ifNone: [
		rPackage := packages at: categoryNameSymbol ifAbsent: [^ self].
		"if no MCPackage exist, we unregister the package from the RPackageOrganizer"
		self unregisterPackageNamed: categoryNameSymbol.	
		].
	
! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrille.delaunay 11/24/2010 12:57'!
systemCategoryRenamedActionFrom: ann

	| rPackage |
	"self flag: 'Seems to be ok.'."
	"we update the rpackage itself with the new name"
	rPackage := self packageNamed: ann oldName asSymbol.
	rPackage name: ann newName.
	
	"we update the organizer"
	packages at: ann newName asSymbol put: rPackage.
	packages removeKey: ann oldName asSymbol
		
! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrille.delaunay 11/26/2010 16:27'!
systemClassAddedActionFrom: ann

	| classNameSymbol class rPackage categoryNameSymbol |
	"self flag: 'seems to be ok'."
	classNameSymbol := ann event item theNonMetaClass name asSymbol.
	class := Smalltalk globals at: classNameSymbol.
	categoryNameSymbol := class category.
	rPackage := packages at: categoryNameSymbol ifAbsent: [
		"a class may be defined for a package that does not yet exist. In this case we have to create first the new package"
		self registerPackage: (RPackage named: categoryNameSymbol).
		].		
	"when we add the class, the parent category should normally already be register correctly at the RPackageOrganizer"
	"this next line should import the class and all its methods: "
	rPackage importClass: class.
	"maybe the last update should be (is?) automatically done when adding or removing from a RPackage."
	"self default registerPackage: rPackage forClass: class"
	"maybe have to check for class and metaClass that will maybe both throw an event?
	=> seems to be correctly handled by RPackage"! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrilledelaunay 1/5/2011 15:41'!
systemClassRecategorizedACtionFrom: ann

	| class newRPackage oldRPackage |
	"self flag: 'seems to be ok'."
	class := ann itemClass.	
	oldRPackage := self packageNamed: ann oldCategory.
	newRPackage := self packageNamed: ann newCategory.
	"we have to update the old package"
	oldRPackage removeClass: class.	
	"we have to update the new package"
	newRPackage importClass: class.	
	
	
	"we have to update the RPackageOrganizer.
	maybe the last update should be automatically done when adding or removing from a RPackage
	"
	"self default unregisterPackage: oldRPackage forClass: class.
	self default registerPackage: newRPackage forClass: class"! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrille.delaunay 11/24/2010 13:21'!
systemClassRemovedActionFrom: ann

	| class rPackage categoryName |
	"self flag: 'seems to be ok'."
	
	"we have to update the RPackageOrganizer  and unregister the class from it"
	class := ann event item.
	categoryName := ann event itemCategory.	
	rPackage := packages at: categoryName ifAbsent: [
		"if the package is not found, we do nothing"
		^ self
		].
	"we have to update the rPackage and unregister the class and all its methods from it"
	rPackage removeClass: class.
	"we have to update the organizer:
             => update the 'classPackageDictionary' to remove the class"
	"maybe the last update should be automatically done when adding or removing from a RPackage => DONE"
	"self default unregisterPackage: rPackage forClass: class"! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrille.delaunay 11/24/2010 13:44'!
systemClassRenamedActionFrom: ann

	| newName oldName categoryName class rPackage extendingRPackages |
	"self flag: 'seems to be ok'."
	class := ann event item.
	categoryName := ann event itemCategory.
	oldName := ann event oldName.
	newName := ann event newName.
	rPackage := self packageNamed: categoryName.
	extendingRPackages := self extendingPackagesOfClassNamed: oldName.
	 	
	"we have to update the rPackage in which the class is defined "
	rPackage updateClassDefinedNamed: oldName withNewName: newName.
	"we have to update all RPackages extending this class"
	extendingRPackages do: [:aRPackage |
		aRPackage updateExtensionForClassNamed: oldName withNewName: newName
		].
		
	"we have to update the RPackageOrganizer.
	update the 'classPackageDictionary' to replace the key with the new class name"
	self unregisterPackage: rPackage forClassName: oldName.
	self registerPackage: rPackage forClassName: newName.	
	"update the 'classExtendingPackagesMapping' to replace the key with the new class name"
	extendingRPackages do: [:aRPackage |
		self unregisterExtendingPackage: aRPackage forClassName: oldName.
		self registerExtendingPackage:  aRPackage forClassName: newName   
		].
	"maybe we should remove alse the key from the extendedPackages dictionary"

	! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'RPackage 1/19/2011 16:11'!
systemClassReorganizedActionFrom: ann
	"when a class is reorganized, we have to check if an extension has not been added"
	
	|class|
	self flag: 'Not sure this method is usefull. If we remove a category, it seems that we will remove the methods inside, therefore the remove method announcement will do the job. Then all other modifications does not impact the packages'.
	"The event ReorganizedEvent is lauched for several cases:
		- sort categories
		- remove a category
		- add a category.
	So the work here is quite big :) The event only provide a reference to the class that has been modified. So I guess we have to:
		=> we check that each extending rPackage contain all methods from the corresponding extension category (as extension methods)
		=> the parentPackage of the class includes all methods defined in classic categories
	"
	class := ann event item.
	class organization categories do: [:aString |
		(aString beginsWith: '*')
			ifTrue: [
				"here we deal with extension-categories"
				|extendingPackageName extendingPackage|
				extendingPackageName := aString copyWithout: $*.
				extendingPackage := self packageMatchingExtensionName: extendingPackageName.
				extendingPackage ifNotNil: [
					"we check that all extension method for this classs in the rpackage correspond to all methods in the extension category"
					(class methodsInCategory: aString) do: [:aMethodsSelector |
						((extendingPackage extensionMethodsForClass: class) includes: (class methodDict at: aMethodsSelector))
							ifFalse: [
								"if a method has not been yet registered in the rPackage, we add it"
								((class methodDict at: aMethodsSelector) packageFromOrganizer: self ) removeMethod: (class methodDict at: aMethodsSelector).
								extendingPackage addMethod: (class methodDict at: aMethodsSelector).
								]  
						]
					]
				]
			ifFalse: [
				"here we deal with classic categories"
				(class methodsInCategory: aString) do: [:aMethodSelector |
					((class package definedMethodsForClass: class) includes: (class methodDict at: aMethodSelector ) )
						ifFalse: [
							((class methodDict at: aMethodSelector) packageFromOrganizer: self) removeMethod: (class methodDict at: aMethodSelector).
							class package addMethod: (class methodDict at: aMethodSelector)
							]
					]
				]
			
		].
	! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'RPackage 1/19/2011 12:16'!
systemMethodAddedActionFrom: ann

	|class selector protocol |
	"self flag: 'seems to be ok'."
	
	class := ann itemClass.
	protocol := ann itemProtocol.
	selector := ann itemSelector.
	"we have to register the method in the parent RPackage of the class. 
	to do that, we first have to look if the method is an extension from an external package:"
	(protocol beginsWith: '*')
		ifTrue: [
			|extendingPackageName rPackage|
			extendingPackageName := protocol copyWithout: $*.
			rPackage := (self packageMatchingExtensionName: extendingPackageName) ifNil: [
				"If the extension name does not match with any registered package, we create a new one, and add the method in this one. This is how monticello does."
				"class package"
				self registerPackage: (RPackage named: extendingPackageName).
				].
			 rPackage addMethod: (class methodDictionary at: selector).
			]
		ifFalse: [
			"if the method is not defined as an extension, we add it to class parentPackage"
			class package addMethod: (class methodDictionary at: selector)
			].
	
	
	! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrilledelaunay 1/26/2011 14:01'!
systemMethodModifiedActionFrom: ann

	"this annoucement can correspond to several kind of modifications:
		=> a method has been renamed
		=> a method has been move to another category (
			-maybe from the classic category to an extending package
			-maybe from an extending package to another extending package
			-maybe from an extending package to a classic category
			- maybe from a classic category to another classic category
			)
	"
	| newProtocol oldProtocol oldMethod newMethod methodPackage destinationPackage |
	self flag: 'see 1) and 2)'.
	newMethod := ann event oldItem.
	oldMethod := ann event item.
	"2) here we have to check that the event is of type ModifiedMethodEvent before sending the message. The event can also be of type ModifiedEvent"
	(ann event isKindOf: ModifiedMethodEvent)
		ifTrue: [
			newProtocol := ann event newProtocol.
			oldProtocol := ann event oldProtocol.
			].
	
	"1) before retrieving the package of the method, we should make sure that the method has already been registered in RPackage. This announcement is emit with the method 'compile:', and for some reasons I don't know yet, it could happen that this announcement  is emitted without having the method already registered. Therefore a 'security test' would avoid this problem, by invoking the add announcement before going on. not yet done"
	methodPackage := newMethod packageFromOrganizer: self.
	"maybe the name of the method has changed"
	oldMethod selector = newMethod selector
		ifFalse: [
			methodPackage updateSelector: oldMethod selector inClass: oldMethod methodClass withNewSelector: newMethod selector
			].
		
	"maybe the category has changed"
	"newProtocol and oldProtocol might be nil here. That is why we have to use asString before"
	newProtocol asString asLowercase = oldProtocol asString asLowercase
		ifFalse: [
			(newProtocol beginsWith: '*') 
				ifTrue: [
					"according the extension specified is valid or not, the method will be defined as an extension of an external package or in the class parent package"
					destinationPackage := (RPackageOrganizer default packageMatchingExtensionName: (newProtocol copyWithout: $*)) ifNil: [newMethod methodClass package]. 
					]
				ifFalse: [
					destinationPackage := newMethod methodClass package.
					].
			"then, we will remove the method from the method package, and add it to the destination package (we check before that the source package and the destination package are different)."
			methodPackage = destinationPackage
				ifFalse: [
					methodPackage removeMethod: oldMethod.
					destinationPackage addMethod: newMethod
					]
			]
	
	! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrilledelaunay 1/26/2011 14:36'!
systemMethodRecategorizedActionFrom: ann

	" a method has been move to another category 
		-maybe from the classic category to an extending package
		-maybe from an extending package to another extending package
		-maybe from an extending package to a classic category
		- maybe from a classic category to another classic category
			"
	|oldProtocol newProtocol method methodPackage destinationPackage|
	self flag: 'see 1)'.
	oldProtocol := ann event oldCategory.
	"old protocol might be nil:"
	oldProtocol ifNil: [oldProtocol := ''].
	newProtocol := ann event itemProtocol.
	method := ann event item.
	"1)Same problem than methodModifiedACtionFrom: . before retrieving the package of the method, we should make sure that the method has already been registered in RPackage. This announcement is emit with the method 'compile:', and for some reasons I don't know yet, it could happen that this announcement  is emitted without having the method already registered. Therefore a 'security test' would avoid this problem, by invoking the add announcement before going on. See compiledMethod >> packageFromOrganizer:"
	"method methodClass packages detect: [:each | each includesSelector: method selector ofClass: method methodClass ] ifNone: [
		^ SystemChangeNotifier uniqueInstance methodAdded: method selector: method selector inProtocol: newProtocol class: method methodClass.
		]. "
	methodPackage := method packageFromOrganizer: self.
	
	"maybe the category has changed"
	newProtocol asLowercase = oldProtocol asLowercase
		ifFalse: [
			(newProtocol beginsWith: '*') 
				ifTrue: [
					"according the extension specified is valid or not, the method will be defined as an extension of an external package or in the class parent package"
					destinationPackage := (self packageMatchingExtensionName:  (newProtocol copyWithout: $*)) ifNil: [method methodClass package]. 
					]
				ifFalse: [
					destinationPackage := method methodClass package.
					].
			"then, we will remove the method from the method package, and add it to the destination package (we check before that the source package and the destination package are different)."
			methodPackage = destinationPackage
				ifFalse: [
					methodPackage removeMethod: method.
					destinationPackage addMethod: method
					]
			]
	
	
	! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrille.delaunay 11/18/2010 11:42'!
systemMethodRemovedActionFrom: ann

	|class protocol selector method methodPackage |
	class := ann itemClass.
	protocol := ann itemProtocol.
	selector := ann itemSelector.
	method := ann event item.
	
	methodPackage := method packageFromOrganizer: self.
	methodPackage removeMethod: method
	
	
	
	! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrille.delaunay 11/18/2010 11:42'!
systemProtocolAddedActionFrom: ann

	"for now I don't see anything to do'"! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrille.delaunay 11/18/2010 11:45'!
systemProtocolRemovedActionFrom: ann

	|protocol class|
	protocol := ann event item.
	class := ann event itemClass.
	
	"If the category is an extension from a package, I would move all the methods concerned, from the extending RPackage to the class RPackage"
	(protocol beginsWith: '*')
		ifTrue: [
			|extendingPackageName destinationPackage |
			extendingPackageName := (protocol copyWithout: $*).
			destinationPackage := self packages at: extendingPackageName ifAbsent: [nil].
			destinationPackage ifNotNil: [
				|methods|
				methods := (destinationPackage methodsForClass: class) copy.
				methods do: [:aCompiledMethod |
					destinationPackage removeMethod: aCompiledMethod.
					class package addMethod: aCompiledMethod
					]
				]
			]
	! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrille.delaunay 11/26/2010 16:05'!
systemTraitDefinitionModifiedActionFrom: ann

	"this kind of anounecement will be used when:
		=> we create a new trait.
		=> we declare new traits to use in a trait"
	|trait|
	trait := ann event item.
	! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrille.delaunay 11/18/2010 13:16'!
unregisterInterestFromAnnouncer: anAnnouncer
	"self registerInterestToSystemAnnouncement"

	"RPackageOrganizer registerInterestToSystemAnnouncement"

	"SystemAnnouncer initialize"

		anAnnouncer
		unsubscribe:  self.! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrille.delaunay 11/18/2010 13:16'!
unregisterInterestToSystemAnnouncement
	"self registerInterestToSystemAnnouncement"

	"RPackageOrganizer registerInterestToSystemAnnouncement"

	"SystemAnnouncer initialize"

		(SystemAnnouncer announcer)
		unsubscribe:  self! !

Object subclass: #PackageChecker
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RPackage-SystemIntegration'!

!PackageChecker methodsFor: 'check' stamp: 'StephaneDucasse 7/30/2010 14:03'!
check
	"self new check"
	
	"only work for top level package overlapping.
	Foo and Foo-Core, but not Foo-Core and Foo-Core-Test"
	| packageNames dict overlappingPackages |
	overlappingPackages := OrderedCollection new.
	packageNames := MCWorkingCopy allManagers collect: [:each | each packageName].
	dict := packageNames groupedBy: [:each | each copyUpTo: $-].
	dict copy keysAndValuesDo: [:k :v | v size = 1 ifTrue: [ dict removeKey: k]].
	dict keysAndValuesDo: [:k :v | (v includes: k) 
									ifTrue: [
										overlappingPackages add: v.
										Warning signal: 'Overlapping packages: ', v printString]].
	^ overlappingPackages ! !

!MCPackageManager class methodsFor: '*rpackage-systemintegration' stamp: 'cyrilledelaunay 1/31/2011 17:17'!
unregisterForNotifications
	Smalltalk globals
		at: #SystemChangeNotifier
		ifPresent:
			[:cls|
			cls uniqueInstance noMoreNotificationsFor: self.]! !

TestCase subclass: #RPackageMCSynchronisationTest
	instanceVariableNames: 'emptyOrganizer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RPackage-SystemIntegration'!

!RPackageMCSynchronisationTest commentStamp: '<historical>' prior: 0!
I had some problems using the 'ensure:' method. There was some conflicts with the organizer used for the test when trying to clean the classes, categories and packages generated for the test. 
There is something I don't get with this method. Therefore I choosed to clean by default all possible generated classes, categories and packages after each test using the tearDown method.

For now, all that was in the 'ensure:' methods has been commented.
Look for example at:
    testAddMethodInClassicCategoryAddMethodToTheParentPackageOfItsClass
and try to uncomment it!

!RPackageMCSynchronisationTest methodsFor: 'running' stamp: 'cyrilledelaunay 1/17/2011 14:28'!
cleanClassesPackagesAndCategories

		|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			Smalltalk removeClassNamed: 'RPackageNewStubClass'.
			Smalltalk removeClassNamed: 'RPackageOldStubClass'.
			Smalltalk removeClassNamed: 'Foo'.
			Smalltalk removeClassNamed: 'FooOther'.
			Smalltalk removeClassNamed: 'NewTrait'.
			Smalltalk organization removeCategory: 'Zork'.
			Smalltalk organization removeCategory: 'XXXXX'.
			Smalltalk organization removeCategory: 'XXXX'.
			Smalltalk organization removeCategory: 'YYYYY'.
			Smalltalk organization removeCategory: 'ZZZZZ'.
			Smalltalk organization removeCategory: 'FooPackage-Core'.
			Smalltalk organization removeCategory: 'FooPackage-Other'.
			Smalltalk organization removeCategory: 'FooPackage'.
			Smalltalk organization removeCategory: 'OriginalCategory'.
			Smalltalk organization removeCategory: 'Y'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'OriginalCategory'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'Yyyyy'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].	
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YyYyY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'Y'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'ZZZZZ'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].	
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'Zzzzz'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'FooPackage-Core'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'FooPackage-Other'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'FooPackage'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'Zork'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			! !

!RPackageMCSynchronisationTest methodsFor: 'private' stamp: 'cyrille.delaunay 11/25/2010 13:17'!
createMethodNamed: methodName inClass: aClass inCategory: aCategoryName 

	^ aClass compile: (methodName, ' ^nil') classified: aCategoryName.! !

!RPackageMCSynchronisationTest methodsFor: 'private' stamp: 'cyrille.delaunay 11/25/2010 13:17'!
createMethodNamed: methodName inClassSideOfClass: aClass inCategory: aCategoryName 

	^ aClass classSide compile: (methodName, ' ^nil') classified: aCategoryName.! !

!RPackageMCSynchronisationTest methodsFor: 'private' stamp: 'StephaneDucasse 7/29/2010 23:46'!
createNewClassNamed: aName
	
	| cls |
	cls := Object subclass: aName
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'RPackageTest'.
	^ cls! !

!RPackageMCSynchronisationTest methodsFor: 'private' stamp: 'cyrille.delaunay 11/22/2010 10:56'!
createNewClassNamed: aName inCategory: aCategoryName
	
	| cls |
	cls := Object subclass: aName asSymbol
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: aCategoryName.
	^ cls! !

!RPackageMCSynchronisationTest methodsFor: 'private' stamp: 'cyrille.delaunay 11/26/2010 15:31'!
createNewTraitNamed: aName inCategory: aCategoryName
	
	| trait |
	trait := Trait named: aName asSymbol
			uses: {}
			category: aCategoryName.
	^ trait! !

!RPackageMCSynchronisationTest methodsFor: 'accessing' stamp: 'RPackage 1/20/2011 13:17'!
emptyOrganizer 

	^ emptyOrganizer! !

!RPackageMCSynchronisationTest methodsFor: 'running' stamp: 'cyrilledelaunay 1/31/2011 17:58'!
setUp 

	"by only using the 'default' RMockEmptyPackageOrganizer, we make sure that each time a new intance of the organizer is created, we unregister safely the last one from teh announcer (see the code of resetDefault). Like that, this old instance will not impact the system anymore when an event will be launched"
	RMockSystemAnnouncer initialize.
	emptyOrganizer := RPackageOrganizer new.
	emptyOrganizer registerPackage: (RPackage named: 'as yet unclassified').
	emptyOrganizer registerInterestToAnnouncer: RMockSystemAnnouncer mockAnnouncer.
	"we also register interest to monticello, to be able to test the synchronisation between it and RPackage"
	emptyOrganizer registerInterestToMC. 
	"we have to unregister Monticello from system events to avoid errors when making our changes. Indeed, by default monticello make some request to the default organizer when changes are made in the system. But This default organizer does not register to system events during our tests"
	MCPackageManager unregisterForNotifications. 
	MCWorkingCopy unregisterForNotifications.
	"This is a second step of security. If by any chance, an Organizer instance persist in the system and is still registered to the (mock)announcer, we also control the (mock)announcer to be active only during the test "
	RMockSystemAnnouncer activateAnnouncerForSystemChanges.
	"As we will create classes in those tests, we set the author name to avoid to have warning when running the tests"
	Author fullName ifNil: [Author fullName: 'RPackage'].! !

!RPackageMCSynchronisationTest methodsFor: 'running' stamp: 'RPackage 1/20/2011 11:50'!
tearDown

	"If after a test, an orgizer is left somewhere, still registered at the mockAnnouncer, we don't want it to impact the system after the test. By disabling the MockSystemAnnouncer, nothing will happen when any kind of event will be lanched from the system"
	RMockSystemAnnouncer disableAnnouncerForSystemChanges.	
	self emptyOrganizer unregisterInterestFromAnnouncer: RMockSystemAnnouncer mockAnnouncer.
	self emptyOrganizer unregisterInterestToMC.
	MCWorkingCopy removeDependent: self emptyOrganizer.
	self cleanClassesPackagesAndCategories.
	Author fullName ifNil: [Author fullName: nil].! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:27'!
testAddCategoryCreatesPackage
	
	"test that when we add a category that does not yet correspond to an existing package, a new package is created"
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer
		do: [ :anOrganizer | 
			Smalltalk organization addCategory: 'XXXXX'.
			self assert: (anOrganizer includesPackageNamed: 'XXXXX').
		]
	] ensure: [
		self toDoAtTheEnd.
		"|mCPackage|
		mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
		mCPackage ifNotNil: [mCPackage unregister].
		Smalltalk organization removeCategory: 'XXXXX'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:27'!
testAddCategoryIsAlreadyAPackageDoesNotCreatePackage
	"test that when we add a category corresponding to an existing package, we do not create a second package"
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ 
			|tmpPackage|
			Smalltalk organization addCategory: 'XXXX'.
			tmpPackage := RPackage organizer packageNamed: #XXXX.
			Smalltalk organization addCategory: 'XXXX'.
			self assert: tmpPackage =  (RPackage organizer packageNamed: #XXXX).
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXX'."
			]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:27'!
testAddClassAddItIntoPackage
	"self debug: #testAddCategoryCreatesPackage"

	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|tmpPackage class|
			Smalltalk organization addCategory: 'XXXXX'.
			self assert: (anOrganizer includesPackageNamed: 'XXXXX').
			tmpPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self assert: (tmpPackage definesClass: class). 
			self assert: (class package = tmpPackage).
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'"
		].
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:27'!
testAddClassInNotCreatedPackage
	"test that when we add a class in an existing category, it does not create a new package"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|tmpPackage class|
			Smalltalk organization addCategory: 'XXXXX'.
			self assert: (anOrganizer includesPackageNamed: 'XXXXX').
			tmpPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			
			self assert: (anOrganizer packageNamed: #XXXXX) = tmpPackage. 
			self assert: (class package = tmpPackage)
			
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'"
		].! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:27'!
testAddClassUpdateTheOrganizer

	"test that when we add a class, the organizer 'classPackageMapping' dictionary is updated, so that the class is linked to its package and so that we can access its parent package"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|tmpPackage class|
			Smalltalk organization addCategory: 'XXXXX'.
			self assert: (anOrganizer includesPackageNamed: 'XXXXX').
			tmpPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self assert: (anOrganizer packageOf: class) = tmpPackage. 
			
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'"
		]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:28'!
testAddMCPackageCreatesAPackage
	"test that when we create a MCPackage, a package corresponding to it is created"
	
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			MCWorkingCopy forPackage: (MCPackage new name: 'Zork').
			self assert: (anOrganizer includesPackageNamed: 'Zork').
			] 
		] ensure: [
			self toDoAtTheEnd.
		"(MCWorkingCopy allManagers detect: [:each | each packageName = 'Zork']) unregister"
		]
		
					  ! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:28'!
testAddMCPackageForCategoryAlreadyExistingDoesNotCreateAPackage
	"test that when we create a MCPackage and that a category of this name already exists, no package is created"
	"self debug: #testAddMCPackageForCategoryAlreadyExistingDoesNotCreateAPackage"
	"self run: #testAddMCPackageForCategoryAlreadyExistingDoesNotCreateAPackage"	
	
	
	| addedCount addedCat | 
	[
	RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|tmpPackage|
			Smalltalk organization addCategory: 'Zork'.
			tmpPackage := anOrganizer packageNamed: #Zork.
			MCWorkingCopy forPackage: (MCPackage new name: 'Zork').
			self assert: tmpPackage = (anOrganizer packageNamed: #Zork).
			] 
		] ensure: [
			self toDoAtTheEnd.
			"Smalltalk organization removeCategory: 'Zork'.
			(MCWorkingCopy allManagers detect: [:each | each packageName = 'Zork']) unregister."
		]
		
		
					  ! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:28'!
testAddMethodInClassicCategoryAddMethodToTheParentPackageOfItsClass

	"test that when we add a method to a  class in a classic category (*not beginning with *), this method is added to the parent package of the class"
	
		| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|tmpPackage class|
			Smalltalk organization addCategory: 'XXXXX'.
			self assert: (anOrganizer includesPackageNamed: 'XXXXX').
			tmpPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass:  class inCategory: 'classic category'.
			self assert: (class package includesDefinedSelector: #stubMethod ofClass: class).
			self assert: ((class >> #stubMethod) packageFromOrganizer: anOrganizer) = class package.
			
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'."
		]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:28'!
testAddMethodInClassicCategoryAddMethodToTheParentPackageOfItsTrait

	"test that when we add a method to a  trait in a classic category (*not beginning with *), this method is added to the parent package of the class"
	
		| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|tmpPackage trait|
			Smalltalk organization addCategory: 'XXXXX'.
			self assert: (anOrganizer includesPackageNamed: 'XXXXX').
			tmpPackage := anOrganizer packageNamed: #XXXXX.
			trait := self createNewTraitNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass:  trait inCategory: 'classic category'.
			self assert: (tmpPackage includesClass: trait).
			self assert: (trait package includesDefinedSelector: #stubMethod ofClass: trait).
			self assert: ((trait >> #stubMethod) packageFromOrganizer: anOrganizer) = trait package.
			
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'."
		]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:28'!
testAddMethodInExtensionCategoryAddMethodToTheExtendingPackage

	"test that when we add a method to a  class in an extension category ( beginning with *), this method is added to the corresponding extending package"
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			| class firstPackage secondPackage|
			Smalltalk organization addCategory: 'XXXXX'.
			Smalltalk organization addCategory: 'YYYYY'.
			firstPackage := anOrganizer packageNamed: #XXXXX.
			secondPackage := anOrganizer packageNamed: #YYYYY.
			self haltIf: [(RPackage organizer =  anOrganizer) not].
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self assert: (class package = firstPackage).
			self haltIf: [(RPackage organizer =  anOrganizer) not].
			self createMethodNamed: 'stubMethod' inClass: class inCategory: '*YYYYY'.
			self haltIf: [(RPackage organizer =  anOrganizer) not].
			self assert: (secondPackage includesExtensionSelector: #stubMethod ofClass: class).
			self deny: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
			self assert: ((class >> #stubMethod) packageFromOrganizer: anOrganizer) = secondPackage.
			
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'.
			Smalltalk organization removeCategory: 'YYYYY'."
		]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:28'!
testAddMethodInExtensionCategoryEnlargingPackageNameAddMethodToTheExtendingPackage

	"test that when we add a method  in an extension category ( beginning with*) that enlarge a package name (for example *mondrian-accessing for Mondrian), this method is added to the corresponding extending package"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			| class XPackage YPackage|
			Smalltalk organization addCategory: 'XXXXX'.
			Smalltalk organization addCategory: 'YYYYY'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self assert: class package = XPackage.
			self createMethodNamed: #newMethod inClass:  class inCategory: '*YYYYY-subcategory'.
			self assert: (YPackage includesExtensionSelector: #newMethod ofClass: class).
			self deny: (XPackage includesDefinedSelector: #newMethod ofClass: class).
			self assert: ((class >> #newMethod) packageFromOrganizer: anOrganizer) = YPackage.
			
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY-subcategory'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'.
			Smalltalk organization removeCategory: 'YYYYY'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'expectedfailures' stamp: 'cyrilledelaunay 1/31/2011 17:29'!
testAddMethodInExtensionCategoryNotExistingCreateANewPackage

	"test that when we add a method  in an extension category ( beginning with *)that does not refer to an existing categorya new package with the name of this category is added, and that the method is added to this new package"
	
		"self flag: 'not sure this is the behavior expected. Maybe a new package should be created and should contain the method. Actually, this is the reason the test fail :)'."
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			| class firstPackage secondPackage|
			Smalltalk organization addCategory: 'XXXXX'.
			Smalltalk organization addCategory: 'YYYYY'.
			firstPackage := anOrganizer packageNamed: #XXXXX.
			secondPackage := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: #newMethod inClass:  class inCategory: '*YYYY'.
			
			self deny: (secondPackage includesExtensionSelector: #newMethod ofClass: class).
			self deny: (class package includesDefinedSelector: #newMethod ofClass: class).
			self assert: ((anOrganizer packageNamed: #YYYY) = nil) not.
			self assert: ((class >> #newMethod) packageFromOrganizer: anOrganizer) = (anOrganizer packageNamed: #YYYY).
			
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'.
			Smalltalk organization removeCategory: 'YYYYY'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:29'!
testAddMethodInExtensionCategoryNotRespectingCaseAddMethodToTheExtendingPackage

	"test that when we add a method  in an extension category ( beginning with *)thae does not match the case of the corresponding package (for example *packagea for PackageA), this method is added to the corresponding extending package"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			| class firstPackage secondPackage|
			Smalltalk organization addCategory: 'XXXXX'.
			Smalltalk organization addCategory: 'YYYYY'.
			firstPackage := anOrganizer packageNamed: #XXXXX.
			secondPackage := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass:  class inCategory: '*yyYyY'.
			self assert: (secondPackage includesExtensionSelector: #stubMethod ofClass: class).
			self deny: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
			self assert: ((class >> #stubMethod) packageFromOrganizer: anOrganizer) = secondPackage.
			
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YyYyY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'.
			Smalltalk organization removeCategory: 'YYYYY'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:29'!
testAddTraitAddItIntoPackage
	"self debug: #testAddCategoryCreatesPackage"

	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|tmpPackage trait|
			Smalltalk organization addCategory: 'XXXXX'.
			self assert: (anOrganizer includesPackageNamed: 'XXXXX').
			tmpPackage := anOrganizer packageNamed: #XXXXX.
			trait := self createNewTraitNamed: 'NewTrait' inCategory: 'XXXXX'.
			self assert: (tmpPackage definesClass: trait). 
			"self assert: (trait package = tmpPackage)."
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'"
		].
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:29'!
testAddTraitInNotCreatedPackage
	"test that when we add a trait in an existing category, it does not create a new package"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|tmpPackage trait|
			Smalltalk organization addCategory: 'XXXXX'.
			self assert: (anOrganizer includesPackageNamed: 'XXXXX').
			tmpPackage := anOrganizer packageNamed: #XXXXX.
			trait := self createNewClassNamed: 'NewTrait' inCategory: 'XXXXX'.
			
			self assert: (anOrganizer packageNamed: #XXXXX) = tmpPackage. 
			self assert: (trait package = tmpPackage)
			
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'"
		].! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:29'!
testAddTraitUpdateTheOrganizer

	"test that when we add a trait, the organizer 'classPackageMapping' dictionary is updated, so that the class is linked to its package and so that we can access its parent package"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|tmpPackage trait|
			Smalltalk organization addCategory: 'XXXXX'.
			self assert: (anOrganizer includesPackageNamed: 'XXXXX').
			tmpPackage := anOrganizer packageNamed: #XXXXX.
			trait := self createNewTraitNamed: 'NewTrait' inCategory: 'XXXXX'.
			self assert: (anOrganizer packageOf: trait) = tmpPackage. 
			
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'"
		]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:29'!
testBasicImportSystemClass
	"self debug: #testBasicImportSystemClass"
	[RPackage 
		withOrganizer: self emptyOrganizer
		do: [ :anOrganizer | 
			| fooClass fooPackage |
			fooClass :=  Object subclass: #Foo
							instanceVariableNames: ''
							classVariableNames: ''
							poolDictionaries: ''
							category: 'FooPackage'.
			fooClass compileSilently: 'foo ^ 1' classified: 'foo'.
			fooClass compileSilently: 'fooBar ^ 2' classified: 'foo'.
			fooClass compileSilently: 'fooFooBar ^ 3' classified: 'foo'.
			fooClass class compileSilently: 'fooClassSide ^ 1' classified: 'foo'.
			fooClass compileSilently: 'zork ^ 4' classified: '*zork'.
			fooPackage := RPackage named: #FooPackage.
			fooPackage importCategory.
			self assert: (fooPackage includesDefinedSelector: #foo ofClass: fooClass).
			self assert: (fooPackage includesDefinedSelector: #fooBar ofClass: fooClass).
			self assert: (fooPackage includesDefinedSelector: #fooFooBar ofClass: fooClass).
			self deny: (fooPackage includesDefinedSelector: #zork ofClass: fooClass).
			self assert: (fooPackage includesDefinedSelector: #fooClassSide ofClass: fooClass class).
			 ]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'Foo'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'FooPackage'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'FooPackage'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:30'!
testCategoryMatching
	
	[self assert: (RPackageOrganizer  default category: 'Compression-Archives' matches: 'Compression').
	self deny: (RPackageOrganizer default category: 'Compression' matches: 'Compression-Archives').
	self assert: (RPackageOrganizer default category: 'Compression' matches: 'Compression').
	self assert: (RPackageOrganizer default category: 'Compression-Arh' matches: 'Compression').
	self deny: (RPackageOrganizer default category: 'XXXX' matches: 'Compression-Archives').
	] ensure: [
		self toDoAtTheEnd.
	]! !

!RPackageMCSynchronisationTest methodsFor: 'expectedfailures' stamp: 'StephaneDucasse 7/30/2010 12:26'!
testCreatePackageWithOtherAround
	"Packages 
		Foo-Core in PackageInfo semantics -> contains classes with Foo-Core
		Foo-Others  in PackageInfo  semantics -> contains classes with Foo-Others
		are created
		
		Foo-Second category is created
		Foo-Second is
		
		"
	
	self flag: #todo.! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:30'!
testImportSystallManagershSubCategoryAndAnotherCategory
	"self debug: #testImportSystemClassWithSubCategoryAndAnotherCategory"
	"indeed the user may have created a class in the category Foo-Core, and after created
	a package only named Foo. The classes in category Foo-Core should be added to Foo. But
	some other packages and categories starting with Foo may exist."
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer | 
			| fooClass fooPackage fooClass2|
			
			fooClass := self createNewClassNamed: 'Foo' inCategory: 'FooPackage-Core'.
			fooClass compileSilently: 'foo ^ 1' classified: 'foo'.
			fooClass compileSilently: 'fooBar ^ 2' classified: 'foo'.
			fooClass compileSilently: 'fooFooBar ^ 3' classified: 'foo'.
			fooClass class compileSilently: 'fooClassSide ^ 1' classified: 'foo'.
			fooClass compileSilently: 'zork ^ 4' classified: '*zork'.
			
			
			fooClass2 := self createNewClassNamed: 'FooOther' inCategory: 'FooPackage-Other'.
			fooClass2 compileSilently: 'fooOther ^ 1' classified: 'what s up'.
			fooClass2 compileSilently: 'fooBarOther ^ 2' classified: 'dr'.
			fooClass2 class compileSilently: 'fooOtherClassSide ^ 1' classified: 'lovely carots'.
			fooClass2 compileSilently: 'zorkZork ^ 4' classified: '*zorkzork'.
			
			fooPackage := RPackage named: #FooPackage.
			fooPackage importCategory.
			self assert: (fooPackage definedClasses size = 2).
			
			self assert: (fooPackage includesDefinedSelector: #foo ofClass: fooClass).
			self assert: (fooPackage includesDefinedSelector: #fooBar ofClass: fooClass).
			self assert: (fooPackage includesDefinedSelector: #fooFooBar ofClass: fooClass).
			self deny: (fooPackage includesDefinedSelector: #zork ofClass: fooClass).
			self assert: (fooPackage includesDefinedSelector: #fooClassSide ofClass: fooClass class).
			
			self assert: (fooPackage includesDefinedSelector: #fooOther ofClass: fooClass2).
			self assert: (fooPackage includesDefinedSelector: #fooBarOther ofClass: fooClass2).
			self deny: (fooPackage includesDefinedSelector: #zorkZork ofClass: fooClass2).
			self assert: (fooPackage includesDefinedSelector: #fooOtherClassSide ofClass: fooClass2 class).
			
			 ]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'Foo'.
			Smalltalk removeClassNamed: 'FooOther'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'FooPackage-Core'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'FooPackage-Other'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'FooPackage-Core'.
			Smalltalk organization removeCategory: 'FooPackage-Other'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:30'!
testImportSystemClassWithSubCategory
	"self debug: #testImportSystemClassWithSubCategory"
	"indeed the user may have created a class in the category Foo-Core, and after created
	a package only named Foo. The classes in category Foo-Core should be added to Foo. But
	some other packages and categories starting with Foo may exist."
	[
	RPackage 
		withOrganizer: self emptyOrganizer 		
		do: [ :anOrganizer | 
			| fooClass fooPackage |
			fooClass :=  Object subclass: #Foo
							instanceVariableNames: ''
							classVariableNames: ''
							poolDictionaries: ''
							category: 'FooPackage-Core'.
			fooClass compileSilently: 'foo ^ 1' classified: 'foo'.
			fooClass compileSilently: 'fooBar ^ 2' classified: 'foo'.
			fooClass compileSilently: 'fooFooBar ^ 3' classified: 'foo'.
			fooClass class compileSilently: 'fooClassSide ^ 1' classified: 'foo'.
			fooClass compileSilently: 'zork ^ 4' classified: '*zork'.
			fooPackage := RPackage named: #FooPackage.
			fooPackage importCategory.
			self assert: (fooPackage includesDefinedSelector: #foo ofClass: fooClass).
			self assert: (fooPackage includesDefinedSelector: #fooBar ofClass: fooClass).
			self assert: (fooPackage includesDefinedSelector: #fooFooBar ofClass: fooClass).
			self deny: (fooPackage includesDefinedSelector: #zork ofClass: fooClass).
			self assert: (fooPackage includesDefinedSelector: #fooClassSide ofClass: fooClass class).
			
			 ]]
		ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'Foo'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'FooPackage-Core'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'FooPackage-Core'."
			]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:30'!
testImportSystemClassWithSubCategoryAndAnotherCategory
	"self debug: #testImportSystemClassWithSubCategoryAndAnotherCategory"
	"indeed the user may have created a class in the category Foo-Core, and after created
	a package only named Foo. The classes in category Foo-Core should be added to Foo. But
	some other packages and categories starting with Foo may exist."
	[
	RPackage 
		withOrganizer: self emptyOrganizer 
		do: [ :anOrganizer | 
			| fooClass fooPackage fooClass2|
			fooClass :=  Object subclass: #Foo
							instanceVariableNames: ''
							classVariableNames: ''
							poolDictionaries: ''
							category: 'FooPackage-Core'.
			fooClass compileSilently: 'foo ^ 1' classified: 'foo'.
			fooClass compileSilently: 'fooBar ^ 2' classified: 'foo'.
			fooClass compileSilently: 'fooFooBar ^ 3' classified: 'foo'.
			fooClass class compileSilently: 'fooClassSide ^ 1' classified: 'foo'.
			fooClass compileSilently: 'zork ^ 4' classified: '*zork'.
			
			fooClass2 :=  Object subclass: #FooOther
							instanceVariableNames: ''
							classVariableNames: ''
							poolDictionaries: ''
							category: 'FooPackage-Other'.
			fooClass2 compileSilently: 'fooOther ^ 1' classified: 'what s up'.
			fooClass2 compileSilently: 'fooBarOther ^ 2' classified: 'dr'.
			fooClass2 class compileSilently: 'fooOtherClassSide ^ 1' classified: 'lovely carots'.
			fooClass2 compileSilently: 'zorkZork ^ 4' classified: '*zorkzork'.
			
			fooPackage := RPackage named: #FooPackage.
			fooPackage importCategory.
			self assert: (fooPackage definedClasses size = 2).
			
			self assert: (fooPackage includesDefinedSelector: #foo ofClass: fooClass).
			self assert: (fooPackage includesDefinedSelector: #fooBar ofClass: fooClass).
			self assert: (fooPackage includesDefinedSelector: #fooFooBar ofClass: fooClass).
			self deny: (fooPackage includesDefinedSelector: #zork ofClass: fooClass).
			self assert: (fooPackage includesDefinedSelector: #fooClassSide ofClass: fooClass class).
			
			self assert: (fooPackage includesDefinedSelector: #fooOther ofClass: fooClass2).
			self assert: (fooPackage includesDefinedSelector: #fooBarOther ofClass: fooClass2).
			self deny: (fooPackage includesDefinedSelector: #zorkZork ofClass: fooClass2).
			self assert: (fooPackage includesDefinedSelector: #fooOtherClassSide ofClass: fooClass2 class).
			
			 ]] ensure: [
			self toDoAtTheEnd.
				"|mCPackage|
			Smalltalk removeClassNamed: 'Foo'.
			Smalltalk removeClassNamed: 'FooOther'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'FooPackage-Core'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'FooPackage-Other'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'FooPackage-Core'.
			Smalltalk organization removeCategory: 'FooPackage-Other'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:30'!
testModifyMethodByChangingCode

	"test that when we modify the code the code of a method, everything work well "
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			| class firstPackage secondPackage|
			Smalltalk organization addCategory: 'XXXXX'.
			firstPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass:  class inCategory: 'classic category'.
		
			self assert: (class >> #stubMethod) category = 'classic category'. 
			self assert: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
			self deny: (firstPackage includesExtensionSelector: #stubMethod ofClass: class).
			self assert: ((class >> #stubMethod) packageFromOrganizer: anOrganizer) = firstPackage.
			
			class compile: 'stubMethod ^22222222222'.
			
			"nothing should change"
			self assert: (class >> #stubMethod) category = 'classic category'. 
			self assert: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
			self deny: (firstPackage includesExtensionSelector: #stubMethod ofClass: class).
			self assert: ((class >> #stubMethod) packageFromOrganizer: anOrganizer) = firstPackage.
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'"
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:31'!
testModifyMethodByMovingFromClassicCategoryToClassicCategoryDoesNothing

	"test that when we move a method from a classic category (not begining with *) to another classic category , nothing is modified"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			| class firstPackage secondPackage|
			Smalltalk organization addCategory: 'XXXXX'.
			firstPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass:  class inCategory: 'classic category'.
		
			self assert: (class >> #stubMethod) category = 'classic category'. 
			self assert: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
			self deny: (firstPackage includesExtensionSelector: #stubMethod ofClass: class).
			self assert: ((class >> #stubMethod) packageFromOrganizer: anOrganizer) = firstPackage.
			
			class organization classify: #stubMethod under: 'new category'.
			"class compileSilently: (class>>#stubMethod) getSource classified: 'new category'."
			
			self assert: (class >> #stubMethod) category = 'new category'. 
			self assert: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
			self deny: (firstPackage includesExtensionSelector: #stubMethod ofClass: class).
			self assert: ((class >> #stubMethod) packageFromOrganizer: anOrganizer) = firstPackage.
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'"
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:31'!
testModifyMethodByMovingFromClassicCategoryToExtensionCategoryMoveItFromParentPackageToExtendingPackage

	"test that when we move a method from a classic category (not begining with *) to an extension category , the method is moved from the parent package of the class to the extending package"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			| class firstPackage secondPackage|
			Smalltalk organization addCategory: 'XXXXX'.
			Smalltalk organization addCategory: 'YYYYY'.
			firstPackage := anOrganizer packageNamed: #XXXXX.
			secondPackage := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass:  class inCategory: 'classic category'.
		
			self assert: (class >> #stubMethod) category = 'classic category'. 
			self assert: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
			self deny: (firstPackage includesExtensionSelector: #stubMethod ofClass: class).
			self assert: ((class >> #stubMethod) packageFromOrganizer: anOrganizer) = firstPackage.
			
			class organization classify: #stubMethod under: '*yyyyy'.
			"class compileSilently: (class>>#stubMethod) getSource classified: '*yyyyy'."
			
			self assert: (class >> #stubMethod) category = '*yyyyy'. 
			self deny: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
			self assert: (secondPackage includesExtensionSelector: #stubMethod ofClass: class).
			self assert: ((class >> #stubMethod) packageFromOrganizer: anOrganizer) = secondPackage.
			
			class organization classify: #stubMethod under: '*yyyyy-suncategory'.
			"class compileSilently: (class>>#stubMethod) getSource classified: '*yyyyy-suncategory'."
			self assert: (class >> #stubMethod) category = '*yyyyy-suncategory'. 
			self assert: ((class >> #stubMethod) packageFromOrganizer: anOrganizer) = secondPackage.
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'Yyyyy'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'.
			Smalltalk organization removeCategory: 'YYYYY'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:31'!
testModifyMethodByMovingFromExtensionCategoryToClassicCategoryMoveItFromExtendingPackageToParentPackage

	"test that when we move a method from an extension category ( begining with *) to a classic category , the method is moved from the extending package to the parent package of the class"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			| class firstPackage secondPackage|
			Smalltalk organization addCategory: 'XXXXX'.
			Smalltalk organization addCategory: 'YYYYY'.
			firstPackage := anOrganizer packageNamed: #XXXXX.
			secondPackage := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass:  class inCategory: '*yyyyy'.
		
			self assert: (class >> #stubMethod) category = '*yyyyy'. 
			self deny: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
			self assert: (secondPackage includesExtensionSelector: #stubMethod ofClass: class).
			self assert: ((class >> #stubMethod) packageFromOrganizer: anOrganizer) = secondPackage.
			
			class organization classify: #stubMethod under: 'classic category'.
			
			self assert: (class >> #stubMethod) category = 'classic category'. 
			self assert: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
			self deny: (secondPackage includesExtensionSelector: #stubMethod ofClass: class).
			self assert: ((class >> #stubMethod) packageFromOrganizer: anOrganizer) = firstPackage.
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'Yyyyy'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'.
			Smalltalk organization removeCategory: 'YYYYY'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:31'!
testModifyMethodByMovingFromExtensionCategoryToExtensionCategoryMoveItFromExtendingPackageToSecondExtendingPackage

	"test that when we move a method from an extension category ( begining with *) to another extending package , the method is moved from the extending package to the other extending package"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			| class XPackage YPackage ZPackage|
			Smalltalk organization addCategory: 'XXXXX'.
			Smalltalk organization addCategory: 'YYYYY'.
			Smalltalk organization addCategory: 'ZZZZZ'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			ZPackage := anOrganizer packageNamed: #ZZZZZ.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: #newMethod inClass:  class inCategory: '*yyyyy'.
		
			self assert: (class >> #newMethod) category = '*yyyyy'. 
			self deny: (XPackage includesDefinedSelector: #newMethod ofClass: class).
			self assert: (YPackage includesExtensionSelector: #newMethod ofClass: class).
			self assert: ((class >> #newMethod) packageFromOrganizer: anOrganizer) = YPackage.
			
			class organization classify: #newMethod under: '*zzzzz'.
			
			self assert: (class >> #newMethod) category = '*zzzzz'. 
			self deny: (XPackage includesDefinedSelector: #newMethod ofClass: class).
			self deny: (YPackage includesExtensionSelector: #newMethod ofClass: class).
			self assert: (ZPackage includesExtensionSelector: #newMethod ofClass: class).
			self assert: ((class >> #newMethod) packageFromOrganizer: anOrganizer) = ZPackage.
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'ZZZZZ'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'Yyyyy'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'Zzzzz'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'.
			Smalltalk organization removeCategory: 'YYYYY'.
			Smalltalk organization removeCategory: 'ZZZZZ'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:31'!
testModifyMethodByRenamingItUpdateThePackageByReplacingSelectorOfTheMethod

	"test that when we rename a method , the Package in which the method is defined is updated with the new selector of the method and that the old one has been removed"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			| class XPackage refactoring|
			Smalltalk organization addCategory: 'XXXXX'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'rpackageOldStubMethod' asSymbol inClass:  class inCategory: 'classic category'.

			self assert: (class package = XPackage).
			self assert: (XPackage includesDefinedSelector: 'rpackageOldStubMethod' asSymbol ofClass: class).
	
			"ChangeMethodNameRefactoring new renameMethod: #stubMethod 
				in: class
				to: #newMethod
				permutation: (0 to: 0)."
			refactoring := RenameMethodRefactoring renameMethod: 'rpackageOldStubMethod' asSymbol in: class to: #rpackageNewStubMethod permutation: (#()).
			refactoring execute.
			"RBParser parseExpression: refactoring storeString."
		
		
			self assert: (class methodDict includesKey: #rpackageNewStubMethod).	
			self deny: (XPackage includesDefinedSelector: 'rpackageOldStubMethod' asSymbol ofClass: class).
			self assert: (XPackage includesDefinedSelector: #rpackageNewStubMethod ofClass: class).
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:31'!
testRecategorizeClassRegisterTheClassInTheNewPackage

	"test that when we recategorize a class, the new package in which it is defined include it"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage YPackage class|
			Smalltalk organization addCategory: 'XXXXX'.
			Smalltalk organization addCategory: 'YYYYY'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self assert: (anOrganizer packageOf: class) = XPackage. 
			
			class category: 'YYYYY'.
			self assert: (anOrganizer packageOf: class) = YPackage. 
			self deny: (anOrganizer packageOf: class) = XPackage. 
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'.
			Smalltalk organization removeCategory: 'YYYYY'."
		]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:31'!
testRecategorizeClassRegisterTheClassMethodsInTheNewPackage

	"test that when we recategorize a class, the new package in which it is defined include all the methods defined in this class (not extensions)"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage YPackage ZPackage class|
			Smalltalk organization addCategory: 'XXXXX'.
			Smalltalk organization addCategory: 'YYYYY'.
			Smalltalk organization addCategory: 'ZZZZZ'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			ZPackage := anOrganizer packageNamed: #ZZZZZ.
			
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'method1' inClass: class inCategory: 'category'.
			self createMethodNamed: 'method2' inClass: class inCategory: 'category'.
			self createMethodNamed: 'method3' inClass: class inCategory: '*zzzzz'.
			self createMethodNamed: 'method4' inClass: class inCategory: '*yyyyy'.
			
			self assert: (XPackage includesDefinedSelector: #method1 ofClass: class ). 
			self assert: (XPackage includesDefinedSelector: #method2 ofClass: class ).
			self assert: (ZPackage includesExtensionSelector: #method3 ofClass: class ). 
			self assert: (YPackage includesExtensionSelector: #method4 ofClass: class ). 
			
			class category: 'YYYYY'.
			
			self assert: (YPackage includesDefinedSelector: #method1 ofClass: class ). 
			self deny: (XPackage includesDefinedSelector: #method1 ofClass: class ). 
			self deny: (XPackage includesExtensionSelector: #method1 ofClass: class ). 
			self assert: (YPackage includesDefinedSelector: #method2 ofClass: class ).
			self deny: (XPackage includesDefinedSelector: #method2 ofClass: class ). 
			self deny: (XPackage includesExtensionSelector: #method2 ofClass: class ). 
			self deny: (YPackage includesExtensionSelector: #method3 ofClass: class ). 
			self deny: (YPackage includesDefinedSelector: #method3 ofClass: class ). 
			self assert: (ZPackage includesExtensionSelector: #method3 ofClass: class ). 
			"I am not sure about this test. For now it does not pass, but I don't know what is the expecting behavior here."
			"self assert: (YPackage includesExtensionSelector: #method4 ofClass: class ). "
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'ZZZZZ'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'Yyyyy'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'Zzzzz'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'.
			Smalltalk organization removeCategory: 'YYYYY'.
			Smalltalk organization removeCategory: 'ZZZZZ'."
		]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:31'!
testRecategorizeClassUnregisterTheClassFromTheOldPackage

	"test that when we recategorize a class, the old package in which it was defined don't include it anymore"
	
		| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage YPackage class|
			Smalltalk organization addCategory: 'XXXXX'.
			Smalltalk organization addCategory: 'YYYYY'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self assert: (anOrganizer packageOf: class) = XPackage. 
			
			class category: 'YYYYY'.
			self deny: (XPackage includesClass: class). 
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'.
			Smalltalk organization removeCategory: 'YYYYY'."
		]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:31'!
testRecategorizeClassUnregisterTheClassMethodsFromTheOldPackage

	"test that when we recategorize a class, the old package in which it was defined don't include its defined methods (not extensions) anymore"
	"self debug: #testRecategorizeClassUnregisterTheClassMethodsFromTheOldPackage"
		
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage YPackage ZPackage class|
			Smalltalk organization addCategory: 'XXXXX'.
			Smalltalk organization addCategory: 'YYYYY'.
			Smalltalk organization addCategory: 'ZZZZZ'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			ZPackage := anOrganizer packageNamed: #ZZZZZ.
			
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'method1' inClass: class inCategory: 'category'.
			self createMethodNamed: 'method2' inClass: class inCategory: 'category'.
			self createMethodNamed: 'method3' inClass: class inCategory: '*zzzzz'.
			self createMethodNamed: 'method4' inClass: class inCategory: '*yyyyy'.
			
			self assert: (XPackage includesDefinedSelector: #method1 ofClass: class ). 
			self assert: (XPackage includesDefinedSelector: #method2 ofClass: class ).
			self assert: (ZPackage includesExtensionSelector: #method3 ofClass: class ). 
			self assert: (YPackage includesExtensionSelector: #method4 ofClass: class ). 
			
			class category: 'YYYYY'.
			
		 
			self deny: (XPackage includesDefinedSelector: #method1 ofClass: class ). 
			self deny: (XPackage includesExtensionSelector: #method1 ofClass: class ). 
			self deny: (XPackage includesDefinedSelector: #method2 ofClass: class ). 
			self deny: (XPackage includesExtensionSelector: #method2 ofClass: class ). 
			self deny: (XPackage includesDefinedSelector: #method3 ofClass: class ). 
			self deny: (XPackage includesExtensionSelector: #method3 ofClass: class ). 
			self deny: (XPackage includesDefinedSelector: #method4 ofClass: class ). 
			self deny: (XPackage includesExtensionSelector: #method4 ofClass: class ).
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'ZZZZZ'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'Zzzzz'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'Yyyyy'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].						
			Smalltalk organization removeCategory: 'XXXXX'.
			Smalltalk organization removeCategory: 'YYYYY'.
			Smalltalk organization removeCategory: 'ZZZZZ'."
		]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:31'!
testRecategorizeClassUpdateTheOrganizer

	"test that when we recategorize a class, the organizer is updated so that the class name point the the new RPackage"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage YPackage class|
			Smalltalk organization addCategory: 'XXXXX'.
			Smalltalk organization addCategory: 'YYYYY'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
		
			class category: 'YYYYY'.
			
			self assert: (anOrganizer packageOf: class) = YPackage
		
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'.
			Smalltalk organization removeCategory: 'YYYYY'."
		]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:31'!
testRecategorizeClassWithExtensionInTheExtendedPackage

	"test that when we recategorize a class with extension methods in the corresponding extended package, the extension methods become defined methods"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage YPackage class|
			Smalltalk organization addCategory: 'XXXXX'.
			Smalltalk organization addCategory: 'YYYYY'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: #RPackageTestsExtensionMethod inClass: class  inCategory: '*yyyyy'.  
			self assert: (anOrganizer packageOf: class) = XPackage. 
			self assert: (YPackage extensionSelectors includes: #RPackageTestsExtensionMethod). 
			self deny: ((XPackage definedSelectorsForClass: class) includes: #RPackageTestsExtensionMethod). 
			
			
			class category: 'YYYYY'.
			self assert: (anOrganizer packageOf: class) = YPackage. 
			self deny: (anOrganizer packageOf: class) = XPackage. 
			self deny: ((XPackage definedSelectorsForClass: class) includes: #RPackageTestsExtensionMethod).
			self assert: ((YPackage definedSelectorsForClass: class)  includes: #RPackageTestsExtensionMethod). 
			self deny: (YPackage extensionSelectors includes: #RPackageTestsExtensionMethod). 
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'.
			Smalltalk organization removeCategory: 'YYYYY'."
		]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:32'!
testRemoveCategoryRemoveThePackage

	"test that when we remove a category, the corresponding RPackage is removed from the organizer"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage class|
			Smalltalk organization addCategory: 'XXXXX'.
			self assert: (anOrganizer includesPackageNamed: #XXXXX).
			Smalltalk organization removeCategory: 'XXXXX'.
			self deny: (anOrganizer includesPackageNamed: #XXXXX).
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk organization removeCategory: 'XXXXX'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister]."
		]
		
	! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 2/1/2011 10:20'!
testRemoveCategoryWhenMCPackageExistDoesNotRemoveThePackage

	"test that when we remove a category (usually from the class browser), and that a corresponding MCPackage exist, the RPackage still exist till the MCPackage has been unloaded"
	
	| addedCount addedCat mCPackage | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage class|
			"let's create the category and the MCPackage"
			Smalltalk organization addCategory: 'XXXXX'.
			mCPackage := MCWorkingCopy forPackage: (MCPackage new name: 'XXXXX').
			"we make sure that the RPackage has been create (only one time)"
			self assert: (anOrganizer includesPackageNamed: #XXXXX).
			self assert: (anOrganizer packages select: [:each | each name = 'XXXXX']) size = 1.
			
			"we remove the category"
			Smalltalk organization removeCategory: 'XXXXX'.
			"we make sure the RPackage still exist"
			self assert: (anOrganizer includesPackageNamed: #XXXXX).
			
			"for now, this no longer the expected behavior. See comment in RPackageOrganizer>>update:"
			"mCPackage unregister.
			self deny: (anOrganizer includesPackageNamed: #XXXXX)."
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk organization removeCategory: 'XXXXX'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister]."
		]
		
	! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:32'!
testRemoveClassUnregisterTheClassDefinedMethodsFromItsPackage

	"test that when we remove a class, the class methods defined in the parent package (not extensions) are all removed  from its parent Package"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage  class|
			Smalltalk organization addCategory: 'XXXXX'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: 'classic category'. 
			
			self assert: (XPackage includesDefinedSelector: #stubMethod ofClass: class). 
			Smalltalk removeClassNamed: 'NewClass'. 
			self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class). 
			self deny: (XPackage includesExtensionSelector: #stubMethod ofClass: class). 
			self deny: (XPackage includesSelector: #stubMethod ofClass: class). 
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:32'!
testRemoveClassUnregisterTheClassExtensionMethodsFromTheCorrespondingPackage

	"test that when we remove a class, the class extension methods are all removed  from theire corresponding parent Package"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage YPackage class|
			Smalltalk organization addCategory: 'XXXXX'.
			Smalltalk organization addCategory: 'YYYYY'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'. 
			
			self assert: (YPackage includesExtensionSelector: #stubMethod ofClass: class). 
			Smalltalk removeClassNamed: 'NewClass'. 
			self deny: (YPackage includesDefinedSelector: #stubMethod ofClass: class). 
			self deny: (YPackage includesExtensionSelector: #stubMethod ofClass: class). 
			self deny: (YPackage includesSelector: #stubMethod ofClass: class). 
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'Yyyyy'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].	
			Smalltalk organization removeCategory: 'XXXXX'.
			Smalltalk organization removeCategory: 'YYYYY'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:32'!
testRemoveClassUnregisterTheClassFromItsPackage

	"test that when we remove a class, the class is removed from its parent Package"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage  class|
			Smalltalk organization addCategory: 'XXXXX'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			
			self assert: (XPackage includesClass: class). 
			Smalltalk removeClassNamed: 'NewClass'. 
			self deny: (XPackage includesClass: class). 
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:32'!
testRemoveClassUpdateTheorganizer

	"test that when we remove a class, the organizer is updated so that the class is no longer present in the  'classPackageDictionary' dictionary"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage  class|
			Smalltalk organization addCategory: 'XXXXX'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			
			self assert: (anOrganizer includesClass: class). 
			Smalltalk removeClassNamed: 'NewClass'. 
			self deny: (anOrganizer includesClass: class). 
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:33'!
testRemoveExtensionMethodRemoveMethodFromItsPackage

	"test that when we remove a method that is an extension from an external package, the method is removed from this package"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage  class YPackage|
			Smalltalk organization addCategory: 'XXXXX'.
			Smalltalk organization addCategory: 'YYYYY'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage  := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'. 
			
			self assert: (YPackage includesExtensionSelector: #stubMethod ofClass: class). 
			class removeSelector: #stubMethod.
			self deny: (YPackage includesDefinedSelector: #stubMethod ofClass: class). 
			self deny: (YPackage includesExtensionSelector: #stubMethod ofClass: class). 
			self deny: (YPackage includesSelector: #stubMethod ofClass: class). 
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'Yyyyy'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].	
			Smalltalk organization removeCategory: 'XXXXX'.
			Smalltalk organization removeCategory: 'YYYYY'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 2/1/2011 10:20'!
testRemoveMCPackageRemoveThePackage

	"test that when we remove a MC Package, the corresponding RPackage is removed from the organizer"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage class|
			MCWorkingCopy forPackage: (MCPackage new name: 'XXXXX').
			self assert: (anOrganizer includesPackageNamed: #XXXXX).
			(MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX']) unregister.
			"for now, this no longer the expected behavior. See comment in RPackageOrganizer>>update:"
			"self deny: (anOrganizer includesPackageNamed: #XXXXX)."
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'."
		]
		
	! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:33'!
testRemoveMethodRemoveMethodFromItsPackage

	"test that when we remove a method, the method is remod from the package in which the method is defined"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage  class|
			Smalltalk organization addCategory: 'XXXXX'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: 'classic category'. 
			
			self assert: (XPackage includesDefinedSelector: #stubMethod ofClass: class). 
			class removeSelector: #stubMethod.
			self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class). 
			self deny: (XPackage includesExtensionSelector: #stubMethod ofClass: class). 
			self deny: (XPackage includesSelector: #stubMethod ofClass: class). 
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:33'!
testRenameCategoryChangeTheNameOfThePackage

	"test that when we rename a category, the RPackage corresponding is updated with this new name"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer
		do: [ :anOrganizer |
			|XPackage| 
			Smalltalk organization addCategory: 'XXXXX'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			
			Smalltalk organization renameCategory: 'XXXXX' toBe: 'YYYYY'.
			self assert: (XPackage name = 'YYYYY').
		]
	] ensure: [
			self toDoAtTheEnd.
		"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'.
			Smalltalk organization removeCategory: 'YYYYY'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:33'!
testRenameCategoryUpdateTheOrganizer

	"test that when we rename a category, the organizer dictionary is update with this new name, so that we can access the package with this new name as key"
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer
		do: [ :anOrganizer |
			|XPackage| 
			Smalltalk organization addCategory: 'XXXXX'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			
			Smalltalk organization renameCategory: 'XXXXX' toBe: 'YYYYY'.
			self assert: (anOrganizer packageNamed: 'YYYYY' asSymbol) = XPackage.
			self deny: (anOrganizer includesPackageNamed: #XXXXX).
		]
	] ensure: [
			self toDoAtTheEnd.
		"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'.
			Smalltalk organization removeCategory: 'YYYYY'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:33'!
testRenameClassUpdateClassDefinedInTheParentPackage

	"test that when we rename a class, the classes Defined in the parent package includes the new name"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage  class refactoring|
			Smalltalk organization addCategory: 'XXXXX'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'RPackageOldStubClass' inCategory: 'XXXXX'.
			
			refactoring := RenameClassRefactoring rename: class to: 'RPackageNewStubClass'.
			refactoring execute.
			
			self assert: (XPackage includesClassNamed: 'RPackageNewStubClass'). 
			self deny: (XPackage includesClassNamed: 'RPackageOldStubClass'). 
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'RPackageNewStubClass'.
			Smalltalk removeClassNamed: 'RPackageOldStubClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:33'!
testRenameClassUpdateClassDefinedSelectorsInTheParentPackage

	"test that when we rename a class, the  'classDefinedSelectors' dictionary of the parent package is updated with the new name. There fore we test that we can correctly access the selector from the package by specifying the right name (the new name)"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage  class refactoring|
			Smalltalk organization addCategory: 'XXXXX'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'RPackageOldStubClass' asSymbol inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: 'classic category'.
			
			refactoring := RenameClassRefactoring rename: class to: 'RPackageNewStubClass'.
			refactoring execute.
			
			self assert: (XPackage includesDefinedSelector: #stubMethod ofClassName: 'RPackageNewStubClass' asSymbol ). 
			self deny: (XPackage includesDefinedSelector: #stubMethod ofClassName: 'RPackageOldStubClass' asSymbol ).
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'RPackageNewStubClass'.
			Smalltalk removeClassNamed: 'RPackageOldStubClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:34'!
testRenameClassUpdateClassExtensionSelectorsInTheExtendingPackages

	"test that when we rename a class, the  'classExtensionSelectors' dictionaries of the extending packages  are updated with the new name. Therfore we test that we can correctly access the selectors from the package by specifying the right name (the new name)"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage YPackage class refactoring|
			Smalltalk organization addCategory: 'XXXXX'.
			Smalltalk organization addCategory: 'YYYYY'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'RPackageOldStubClass' asSymbol inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'.
			
			refactoring := RenameClassRefactoring rename: class to: 'RPackageNewStubClass'.
			refactoring execute.
			
			self assert: (YPackage includesExtensionSelector: #stubMethod ofClassName: 'RPackageNewStubClass' asSymbol ). 
			self deny: (YPackage includesExtensionSelector: #stubMethod ofClassName: 'RPackageOldStubClass' asSymbol ).
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'RPackageNewStubClass'.
			Smalltalk removeClassNamed: 'RPackageOldStubClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'Yyyyy'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].	
			Smalltalk organization removeCategory: 'XXXXX'.
			Smalltalk organization removeCategory: 'YYYYY'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:34'!
testRenameClassUpdateMetaclassDefinedSelectorsInTheParentPackage

	"test that when we rename a class, the  'metaclassDefinedSelectors' dictionary of the parent package is updated with the new name. Ther fore we test that we can correctly access the selector from the package by specifying the right name (the new name)"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage  class refactoring|
			Smalltalk organization addCategory: 'XXXXX'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'RPackageOldStubClass' asSymbol inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClassSideOfClass: class  inCategory: 'classic category'.
			
			refactoring := RenameClassRefactoring rename: class to: 'RPackageNewStubClass'.
			refactoring execute.
			
			self assert: (XPackage  includesDefinedSelector: #stubMethod ofMetaclassName: 'RPackageNewStubClass' asSymbol ). 
			self deny: (XPackage  includesDefinedSelector: #stubMethod ofMetaclassName: 'RPackageOldStubClass' asSymbol ).
			]
		] ensure: [
			
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'RPackageNewStubClass'.
			Smalltalk removeClassNamed: 'RPackageOldStubClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:35'!
testRenameClassUpdateMetaclassExtensionSelectorsInTheExtendingPackages

	"test that when we rename a class, the  'metaclassExtensionSelectors' dictionaries of the extending packages  are updated with the new name. Ther fore we test that we can correctly access the selectors from the package by specifying the right name (the new name)"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage YPackage class refactoring|
			Smalltalk organization addCategory: 'XXXXX'.
			Smalltalk organization addCategory: 'YYYYY'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'RPackageOldStubClass' asSymbol inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClassSideOfClass: class  inCategory: '*yyyyy'.
			
			refactoring := RenameClassRefactoring rename: class to: 'RPackageNewStubClass'.
			refactoring execute.
			
			self assert: (YPackage includesExtensionSelector: #stubMethod ofMetaclassName: 'RPackageNewStubClass' asSymbol ). 
			self deny: (YPackage includesExtensionSelector: #stubMethod ofMetaclassName: 'RPackageOldStubClass' asSymbol ).
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'RPackageNewStubClass'.
			Smalltalk removeClassNamed: 'RPackageOldStubClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'Yyyyy'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].	
			Smalltalk organization removeCategory: 'XXXXX'.
			Smalltalk organization removeCategory: 'YYYYY'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:35'!
testRenameClassUpdateOrganizerClassExtendingPackagesMapping

	"test that when we rename a class, the classExtendingPackages dictionary of the organizer is updated with the new name, so that we can access the packages when specifying the new name"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage YPackage class refactoring|
			Smalltalk organization addCategory: 'XXXXX'.
			Smalltalk organization addCategory: 'YYYYY'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'RPackageOldStubClass' inCategory: 'XXXXX'.
			self createMethodNamed: #stubMethod inClass: class inCategory: '*yyyyy'. 
			
			refactoring := RenameClassRefactoring rename: class to: 'RPackageNewStubClass'.
			refactoring execute.
			
			self assert: ((anOrganizer extendingPackagesOfClassNamed: 'RPackageNewStubClass' asSymbol) includes: YPackage). 
			self deny: ((anOrganizer extendingPackagesOfClassNamed: 'RPackageOldStubClass' asSymbol) includes: YPackage). 
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'RPackageNewStubClass'.
			Smalltalk removeClassNamed: 'RPackageOldStubClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'Yyyyy'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].	
			Smalltalk organization removeCategory: 'XXXXX'.
			Smalltalk organization removeCategory: 'YYYYY'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:35'!
testRenameClassUpdateOrganizerClassPackageMapping

	"test that when we rename a class, the classPackage dictionary of the organizer is updated with the new name, so that we can access the package when specifying the new name"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage class refactoring|
			Smalltalk organization addCategory: 'XXXXX'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'RPackageOldStubClass' inCategory: 'XXXXX'.
			
			refactoring := RenameClassRefactoring rename: class to: 'RPackageNewStubClass'.
			refactoring execute.
			
			self assert: ((anOrganizer packageOfClassNamed: 'RPackageNewStubClass' asSymbol) = XPackage). 
			self assert: ((anOrganizer packageOfClassNamed: 'RPackageOldStubClass' asSymbol) = nil). 
			]
		] ensure: [
			self toDoAtTheEnd.
			"|mCPackage|
			Smalltalk removeClassNamed: 'RPackageNewStubClass'.
			Smalltalk removeClassNamed: 'RPackageOldStubClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:36'!
testReoganizeClassByAddingNewProtocolDoesNothing

	"test that when we reoganized a class by adding a category, nothing change from  RPackage point of vue."
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer
		do: [ :anOrganizer |
			|XPackage YPackage class| 
			Smalltalk organization addCategory: 'XXXXX'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'newMethod' inClass: class inCategory: 'xxxxx'.  
			class organization addCategory: 'accessing' before: nil.
			
			self assert: (XPackage includesClass: class).
			self assert: (XPackage includesDefinedSelector: #newMethod ofClass: class).
			self assert: (XPackage extensionMethods size = 0).
		]
	] ensure: [
		self toDoAtTheEnd.
		"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:36'!
testReoganizeClassByRemovingClassicCategoryRemoveMethodsFromTheParentPackage

	"test that when we reoganized a class by removing a classic category (a category not beginning with '*'), the methods from this category are removed from the parent RPackage of the class"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer
		do: [ :anOrganizer |
			|XPackage YPackage class| 
			Smalltalk organization addCategory: 'XXXXX'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: 'classic category'.
			
			class removeCategory: 'classic category'.
			
			self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class).
		]
	] ensure: [
			self toDoAtTheEnd.
		"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			Smalltalk organization removeCategory: 'XXXXX'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:36'!
testReoganizeClassByRemovingExtensionCategoryRemoveMethodsFromTheExtendingPackage

	"test that when we reoganized a class by removing an extension category (a category beginning with '*'), all the methods that were inside are removed from the extending package"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer
		do: [ :anOrganizer |
			|XPackage YPackage class| 
			Smalltalk organization addCategory: 'XXXXX'.
			Smalltalk organization addCategory: 'YYYYY'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage  := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'.
			
			class removeCategory: '*yyyyy'.
			
			self deny: (YPackage includesExtensionSelector: #stubMethod ofClass: class).
			self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class).
		]
	] ensure: [
			self toDoAtTheEnd.
	"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'Yyyyy'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].	
			Smalltalk organization removeCategory: 'XXXXX'.
			Smalltalk organization removeCategory: 'YYYYY'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:36'!
testReoganizeClassByRenamingClassicCategoryToExtensionCategoryMoveMethodsFromParentPackageToExtendingPackage

	"test that when we reoganized a class by renaming a  classic category (a category not beginning with '*') to an extension category, all the methods are moved from the  parent package of the class to the extending package"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer
		do: [ :anOrganizer |
			|XPackage YPackage class| 
			Smalltalk organization addCategory: 'XXXXX'.
			Smalltalk organization addCategory: 'YYYYY'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage  := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: 'classic category'.
			
			class organization renameCategory: 'classic category' toBe: '*yyyyy'.
			
			self assert: (YPackage includesExtensionSelector: #stubMethod ofClass: class).
			self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class).
		]
	] ensure: [
			self toDoAtTheEnd.
		"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'Yyyyy'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].	
			Smalltalk organization removeCategory: 'XXXXX'.
			Smalltalk organization removeCategory: 'YYYYY'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:37'!
testReoganizeClassByRenamingExtensionCategoryToAnotherExtensionCategoryAddMethodsToTheNewPackage

	"test that when we reoganized a class by renaming an extension category (a category beginning with '*') to another extension category, all the methods are moved to the new extendingPackage"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer
		do: [ :anOrganizer |
			|XPackage YPackage ZPackage class| 
			Smalltalk organization addCategory: 'XXXXX'.
			Smalltalk organization addCategory: 'YYYYY'.
			Smalltalk organization addCategory: 'ZZZZZ'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage  := anOrganizer packageNamed: #YYYYY.
			ZPackage  := anOrganizer packageNamed: #ZZZZZ.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'.
			
			class organization renameCategory: '*yyyyy' toBe: '*zzzzz'.
			
			self assert: (ZPackage includesExtensionSelector: #stubMethod ofClass: class).
			self deny: (YPackage includesExtensionSelector: #stubMethod ofClass: class).
			self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class).
		]
	] ensure: [
			self toDoAtTheEnd.
		"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'ZZZZZ'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'Yyyyy'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].	
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'Zzzzz'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].	
			Smalltalk organization removeCategory: 'XXXXX'.
			Smalltalk organization removeCategory: 'YYYYY'.
			Smalltalk organization removeCategory: 'ZZZZZ'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:37'!
testReoganizeClassByRenamingExtensionCategoryToClassicCategoryMoveMethodsFromExtendingPackageToParentPackage

	"test that when we reoganized a class by renaming an extension category (a category beginning with '*') to a classic category, all the methods are moved from the  extendingPackage to the parent package of the class"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer
		do: [ :anOrganizer |
			|XPackage YPackage class| 
			Smalltalk organization addCategory: 'XXXXX'.
			Smalltalk organization addCategory: 'YYYYY'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage  := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'.
			
			class organization renameCategory: '*yyyyy' toBe: 'classic category'.
			
			self deny: (YPackage includesExtensionSelector: #stubMethod ofClass: class).
			self assert: (XPackage includesDefinedSelector: #stubMethod ofClass: class).
		]
	] ensure: [
			self toDoAtTheEnd.
		"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'Yyyyy'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].	
			Smalltalk organization removeCategory: 'XXXXX'.
			Smalltalk organization removeCategory: 'YYYYY'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'expectedfailures' stamp: 'StephaneDucasse 7/30/2010 11:54'!
testSplitPackage
	"Here the user have created a package Foo which contains several classes some 
	with the category Foo-Core others in category Foo-Others.
	Now it creates if he creates a package Foo-Core, Foo-core should contain the classes
	having Foo-Core as category.
	
	Typically we should have a checker that mentions a problem having Foo and Foo-Core.
	"
	
	self flag: #todo.! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'cyrilledelaunay 1/31/2011 17:37'!
testisDefinedAsPackageOrSubPackageInMC
	"self debug: #testisDefinedAsSubPackageInMC"
	"test whether a given category is defined as a MC package or whether it is defined as 
	top part of package"
	
	"self debug: #testisDefinedAsPackageOrSubPackageInMC"
	
	[self assert: (RPackageOrganizer default isDefinedAsPackageOrSubPackageInMC: #'Compression').
	self deny: (RPackageOrganizer default isDefinedAsPackageOrSubPackageInMC: #'Compression-Archives').
	"Indeed compression-archives is a category but not a MC package"
	
	self assert: (RPackageOrganizer default isDefinedAsPackageOrSubPackageInMC: #'Collections').
	"#Collection is not defined as a package but as a package #'Collections-Unordered'"
	self assert: (RPackageOrganizer default isDefinedAsPackageOrSubPackageInMC: #'Collections-Unordered').
	self deny: (RPackageOrganizer default isDefinedAsPackageOrSubPackageInMC:#'Collections-Unordered-Test').
	] ensure: [
		self toDoAtTheEnd.
	]
	! !

!RPackageMCSynchronisationTest methodsFor: 'running' stamp: 'cyrilledelaunay 1/31/2011 17:59'!
toDoAtTheEnd

	MCPackageManager registerForNotifications.
	MCWorkingCopy registerForNotifications.! !
